#include "grille.h"
#include <Imagine/Graphics.h>
#include <string>
#include <algorithm>
using namespace Imagine;

/// Q2. Dessine les cases de la grille
void dessine_cases() {
    for(int i=0; i<n; i++)
        for(int j=0; j<n; j++)
            drawRect(marge+z*j, marge+z*i, z, z, BLACK);
}

/// Q4. Initialisation de grille
void init(Grille& g) {
    g.score = 0;
    for(int i=0; i<n; i++)
        for(int j=0; j<n; j++)
            g.valeurs[i][j]=0;
    g.i0=g.j0=-1; // Q15
}

/// Q18. Chaque nombre a sa couleur d'affichage
Color couleur(int val) {
    Color palette[7] = {RED, GREEN, BLUE, ORANGE, CYAN, PURPLE, MAGENTA};
    int i=0;
    while(val!=2) {
        val/=2;
        i = (i+1)%7;
    }
    return palette[i];
}

/// Q5. Dessine les nombres dans leur case
void dessine(const Grille& g) {
    dessine_cases();
    for(int i=0; i<n; i++)
        for(int j=0; j<n; j++)
            if(g.valeurs[i][j]!=0) {
                std::string str = std::to_string(g.valeurs[i][j]);
                if(i==g.i0 && j==g.j0) // Q15
                    fillRect(marge+j*z+1, marge+i*z+1, z-2,z-2, YELLOW); 
                drawString(marge+j*z+z/2-str.size()*txt/2,
                           marge+i*z+z/2+txt/2,
                           str, couleur(g.valeurs[i][j]), txt); // Q18
            }
}

/// Q6. Nombre de cases libres
int libres(const Grille& g) {
    int k=0;
    for(int i=0; i<n; i++)
        for(int j=0; j<n; j++)
            if(g.valeurs[i][j]==0)
                ++k;
    return k;
}

/// Q7. Ajoute un 2 ou un 4 dans une case libre
bool ajoute(Grille& g) {
    int k=libres(g);
    if(k==0)
        return false;
    int val=2;
    if(intRandom(0,1) == 1)
        val=4;
    k = intRandom(0,k-1);
    for(int i=0; i<n; i++)
        for(int j=0; j<n; j++)
            if(g.valeurs[i][j]==0)
                if(k==0) {
                    g.valeurs[i][j]=val;
                    g.i0 = i; // Q15
                    g.j0 = j;
                    return true;
                } else
                    --k;
    return true;
}

/// Q9. Mets les zeros a droite dans chaque ligne.
void zeros_a_droite(Grille& g) {
    for(int i=0; i<n; i++) {
        int j=0;
        for(int k=0; k<n; k++) {
            if(g.valeurs[i][k]!=0)
                g.valeurs[i][j++] = g.valeurs[i][k];
        }
        for(; j<n; j++)
            g.valeurs[i][j]=0;
    }
}

/// Q10. Decale les nombres vers la gauche
void decale_gauche(Grille& g) {
    zeros_a_droite(g);
    for(int i=0; i<n; i++) {
        for(int j=0; j+1<n && g.valeurs[i][j+1]!=0; j++) {
            if(g.valeurs[i][j] == g.valeurs[i][j+1]) {
                g.valeurs[i][j] *= 2;
                g.valeurs[i][j+1] = 0;
                g.score += g.valeurs[i][j]; // Q16
                ++j;
            }
            
        }
    }
    zeros_a_droite(g);
}

/// Q11. Transpose chaque ligne
void miroir(Grille & g) {
    for(int i=0; i<n; i++)
        for(int j=0, k=n-1; j<k; j++, k--)
            std::swap(g.valeurs[i][j], g.valeurs[i][k]);
}

/// Q12. Transpose la matrice
void transpose(Grille& g) {
    for(int i=0; i<n; i++)
        for(int j=i+1; j<n; j++)
            std::swap(g.valeurs[i][j], g.valeurs[j][i]);
}

/// Q13. Decale dans la direction indiquee par la touche
void decale(Grille& g, int touche) {
    g.i0=g.j0=-1; // Q15
    if(touche==KEY_UP || touche==KEY_DOWN)
        transpose(g);
    if(touche==KEY_RIGHT || touche==KEY_DOWN)
        miroir(g);
    decale_gauche(g);
    if(touche==KEY_RIGHT || touche==KEY_DOWN)
        miroir(g);
    if(touche==KEY_UP || touche==KEY_DOWN)
        transpose(g);
}
